#include "BattleActSelection.h"
#include "BattleActTargetSelection.h"
#include "BattleMainMenu.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include "utility/misc.h"

#define ACTSELECTION WalrusRPG::Undertale::BattleStates::BattleActSelection

using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;
using WalrusRPG::Undertale::BattleStates::BattleMainMenu;
using WalrusRPG::Undertale::BattleStates::BattleActTargetSelection;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;
using WalrusRPG::Undertale::doable_acts_e;


ACTSELECTION::BattleActSelection(UndertaleBattleStateMachine &machine,
                                 doable_acts_e *acts, unsigned nb_acts)
    : VBattleState(machine), cursor_index(0), acts(acts), nb_acts(nb_acts)
{
}
ACTSELECTION::~BattleActSelection()
{
}

void ACTSELECTION::update(unsigned dt)
{
    UNUSED(dt);
    if (Input::key_pressed(K_DOWN))
        cursor_index = (cursor_index + 2) % nb_acts;
    if (Input::key_pressed(K_UP))
        cursor_index = (cursor_index - 2) % nb_acts;
    if (Input::key_pressed(K_LEFT))
        cursor_index = (cursor_index - 1) % nb_acts;
    if (Input::key_pressed(K_RIGHT))
        cursor_index = (cursor_index + 1) % nb_acts;

    if (Input::key_pressed(K_A))
    {
        machine.get_enemy(machine.selected_enemy_index)
            ->on_act(doable_acts[acts[cursor_index]]);
    }
    if (Input::key_pressed(K_B))
    {
        machine.set_state(new BattleActTargetSelection(machine));
    }
}

void ACTSELECTION::render(unsigned dt)
{
    UNUSED(dt);
    fill(Graphics::Black);

    draw_frame();

    draw_enemies(machine.get_enemies(), machine.get_nb_enemies());

    draw_hud(machine.health, machine.health_max);

    draw_text_hitbox(machine.text_box_dimensions, 2);
    for (unsigned i = 0; i < nb_acts / 2 + 1; i++)
    {
        for (unsigned j = 0; j < 2 && (i * 2 + j < nb_acts); j++)
        {
            put_string("*", 51 + 128 * j, 135 + i * 16);
            put_string(doable_acts[acts[i * 2 + j]].get_name(), 67 + 128 * j,
                       135 + i * 16);
        }
    }

    draw_menu_buttons(1);

    put_sprite(*tex_heart, 32 + 128 * (cursor_index % 2),
               135 + 4 + 16 * (cursor_index / 2), tex_heart->get_dimensions());
}
