#include "BattleActTargetSelection.h"
#include "BattleMainMenu.h"
#include "BattleActSelection.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include "utility/misc.h"

#define ACTTARGETSELECTION WalrusRPG::Undertale::BattleStates::BattleActTargetSelection

using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;
using WalrusRPG::Undertale::BattleStates::BattleMainMenu;
using WalrusRPG::Undertale::BattleStates::BattleActSelection;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;

ACTTARGETSELECTION::BattleActTargetSelection(UndertaleBattleStateMachine &machine)
    : VBattleState(machine), cursor_index(0)
{
}

ACTTARGETSELECTION::~BattleActTargetSelection()
{
}

void ACTTARGETSELECTION::update(unsigned dt)
{
    UNUSED(dt);
    if (Input::key_pressed(K_DOWN))
        cursor_index = (cursor_index + 2) % machine.get_nb_enemies();
    if (Input::key_pressed(K_UP))
        cursor_index = (cursor_index - 2) % machine.get_nb_enemies();
    if (Input::key_pressed(K_LEFT))
        cursor_index = (cursor_index - 1) % machine.get_nb_enemies();
    if (Input::key_pressed(K_RIGHT))
        cursor_index = (cursor_index + 1) % machine.get_nb_enemies();

    if (Input::key_pressed(K_A) && machine.get_enemy(cursor_index)->can_attack())
    {
        machine.selected_enemy_index = cursor_index;
        machine.set_state(new BattleActSelection(
            machine, machine.get_enemy(cursor_index)->get_doable_acts(), 3));
    }
    if (Input::key_pressed(K_B))
        machine.set_state(new BattleMainMenu(machine));
}

void ACTTARGETSELECTION::render(unsigned dt)
{
    UNUSED(dt);
    fill(Graphics::Black);

    draw_frame();

    draw_enemies(machine.get_enemies(), machine.get_nb_enemies());

    draw_hud(machine.health, machine.health_max);

    draw_text_hitbox(machine.text_box_dimensions, 2);

    for (unsigned i = 0; i < machine.get_nb_enemies() / 2 + 1; i++)
    {
        for (unsigned j = 0; j < 2; j++)
        {
            VEnemy *current_enemy = machine.get_enemy(i * 2 + j);
            if (current_enemy == nullptr)
                break;
            if (!current_enemy->can_attack())
            {
                put_string("*", 51 + 128 * j, 135 + i * 16, Gray);
                put_string(current_enemy->get_name(), 67 + 128 * j, 135 + i * 16, Gray);
            }
            else if (current_enemy->is_spareable())
            {
                put_string("*", 51u + 128u * j, 135 + i * 16, Yellow);
                put_string(current_enemy->get_name(), 67 + 128 * j, 135 + i * 16, Yellow);
            }
            else
            {
                put_string("*", 51u + 128u * j, 135 + i * 16);
                put_string(current_enemy->get_name(), 67 + 128 * j, 135 + i * 16);
            }
        }
    }
    draw_menu_buttons(1);

    put_sprite(*tex_heart, 32 + 128 * (cursor_index % 2),
               135 + 4 + 16 * (cursor_index / 2), tex_heart->get_dimensions());
}
