#include "BattleAttack.h"
#include "BattleMainMenu.h"
#include "BattleEnemyAttack.h"
#include "BattleVictory.h"
#include "input/Input.h"
#include "VEnemy.h"
#include "GraphicsEx.h"
#include "utility/misc.h"

using WalrusRPG::Undertale::BattleStates::BattleAttack;
using WalrusRPG::Undertale::BattleStates::BattleEnemyAttack;
using WalrusRPG::Undertale::BattleStates::BattleMainMenu;
using WalrusRPG::Undertale::BattleStates::BattleVictory;
using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;
using namespace WalrusRPG::Undertale;
using WalrusRPG::Input::Key;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;
using WalrusRPG::Undertale::VEnemy;


BattleAttack::BattleAttack(UndertaleBattleStateMachine &machine)
    : VBattleState(machine), aim_bar_position(300), attacked(false),
      life_bar_visibility_timer(0), damage_done(0), previous_hp(0)
{
    machine.text_box_dimensions = {16, 128, 287, 60};
}

BattleAttack::~BattleAttack()
{
}

void BattleAttack::update(unsigned dt)
{
    if (!attacked && !missed)
    {
        aim_bar_position -= 3 * dt;
        if (aim_bar_position < 18)
        {
            missed = true;
            damage_done = 0;
        }

        if (Input::key_pressed(Key::K_A))
        {
            // TODO : calculate damage from attack/def/timing
            attacked = true;
            previous_hp = machine.get_enemy(machine.selected_enemy_index)->get_hp();
            damage_done = 6 - (5 * abs(135 - (signed) aim_bar_position)) / 135;
            machine.get_enemy(machine.selected_enemy_index)->on_attack(damage_done);
        }
        if (attacked || missed)
        {
            life_bar_visibility_timer = 100;
        }
    }
    else
    {
        if (life_bar_visibility_timer <= dt)
        {
            bool genocide = true;
            for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
            {
                if (machine.get_enemy(i)->can_attack())
                {
                    genocide = false;
                    break;
                }
            }
            if (genocide)
            {
                machine.set_state(new BattleVictory(machine));
                return;
            }
            else
                machine.set_state(new BattleEnemyAttack(machine));
        }
        else
            life_bar_visibility_timer -= dt;
    }
}

void BattleAttack::render(unsigned dt)
{
    UNUSED(dt);
    fill(Graphics::Black);
    draw_frame();
    draw_hud(machine.health, machine.health_max);
    draw_text_hitbox(machine.text_box_dimensions, 2);
    unsigned aim_x = (320 - tex_aim->get_dimensions().width) / 2;
    unsigned aim_y = 129 + ((188 - 128) - tex_aim->get_dimensions().height) / 2;
    put_sprite(*tex_aim, aim_x, aim_y, tex_aim->get_dimensions());
    put_rectangle({aim_bar_position, 129, 4, 60}, White);

    if (life_bar_visibility_timer > 0)
    {
        if (life_bar_visibility_timer > 50 ||
            (life_bar_visibility_timer < 50 && life_bar_visibility_timer % 4 == 0))
        {
            VEnemy *current_enemy = machine.get_enemy(machine.selected_enemy_index);
            signed y_damage_offset = 0;
            if (life_bar_visibility_timer >= 80)
            {
                static const signed y_offsets[] = {0,  -1, -2, -2, -3, -3,
                                                   -2, -2, -1, 0,  0};
                y_damage_offset = y_offsets[(life_bar_visibility_timer - 80) / 2];
            }
            if (damage_done > 0)
            {
                if (life_bar_visibility_timer >= 60)
                {
                    unsigned new_hp = current_enemy->get_hp();
                    unsigned decreasing_hp =
                        ((life_bar_visibility_timer - 60) * (previous_hp - new_hp)) / 30;
                    draw_enemy_life_bar(current_enemy->get_hp_max(),
                                        new_hp + decreasing_hp,
                                        current_enemy->get_cell());
                }
                else
                    draw_enemy_life_bar(current_enemy->get_hp_max(),
                                        current_enemy->get_hp(),
                                        current_enemy->get_cell());
            }
            else
                draw_enemy_life_bar(current_enemy->get_hp_max(), current_enemy->get_hp(),
                                    current_enemy->get_cell());
            if (missed)
                draw_enemy_damage_missed(y_damage_offset, current_enemy->get_cell());
            else
                draw_enemy_damage(damage_done, y_damage_offset,
                                  current_enemy->get_cell());
        }
    }
    draw_enemies(machine.get_enemies(), machine.get_nb_enemies());
}