#include "BattleAttackTargetSelection.h"
#include "BattleMainMenu.h"
#include "BattleAttack.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include "utility/misc.h"

#define BATTLETARGET WalrusRPG::Undertale::BattleStates::BattleAttackTargetSelection


using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;
using WalrusRPG::Undertale::BattleStates::BattleMainMenu;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;


BATTLETARGET::BattleAttackTargetSelection(UndertaleBattleStateMachine &machine,
                                          VBattleState *next_state)
    : VBattleState(machine), cursor(0), next_state(next_state)
{
}

BATTLETARGET::~BattleAttackTargetSelection()
{
}

void BATTLETARGET::update(unsigned dt)
{
    UNUSED(dt);
    if (Input::key_pressed(K_DOWN))
        cursor = (cursor + 1) % machine.get_nb_enemies();
    if (Input::key_pressed(K_UP))
        cursor = (cursor - 1) % machine.get_nb_enemies();
    if (Input::key_pressed(K_A) && machine.get_enemy(cursor)->can_attack())
    {
        machine.selected_enemy_index = cursor;
        machine.set_state(new BattleAttack(machine));
    }
    if (Input::key_pressed(K_B))
    {
        machine.set_state(new BattleMainMenu(machine));
    }
}

void BATTLETARGET::render(unsigned dt)
{
    UNUSED(dt);
    fill(Graphics::Black);

    draw_frame();

    draw_enemies(machine.get_enemies(), machine.get_nb_enemies());

    draw_hud(machine.health, machine.health_max);

    draw_text_hitbox(machine.text_box_dimensions, 2);

    for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
    {
        VEnemy *enemy = machine.get_enemy(i);
        if (!enemy->can_attack())
        {
            put_string("*", 51, 135 + i * 16, Gray);
            put_string(enemy->get_name(), 67, 135 + i * 16, Gray);
        }
        else if (enemy->is_spareable())
        {
            put_string("*", 51, 135 + i * 16, Yellow);
            put_string(enemy->get_name(), 67, 135 + i * 16, Yellow);
        }
        else
        {
            put_string("*", 51, 135 + i * 16);
            put_string(enemy->get_name(), 67, 135 + i * 16);
        }
        if (!enemy->is_spared() && enemy->should_show_lifebar_in_menu())
        {
            put_rectangle({179, 137 + 16 * static_cast<int>(i), 77, 12}, Red);
            put_rectangle({179, 137 + 16 * static_cast<int>(i),
                            static_cast<unsigned>(77 * (enemy->get_hp() / (float) enemy->get_hp_max())),
                           12},
                          Green);
        }
    }

    draw_menu_buttons(0);

    put_sprite(*tex_heart, 32, 135 + 4 + 16 * cursor, tex_heart->get_dimensions());
}
