#include "BattleEncounter.h"
#include "BattleMainMenu.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include "utility/misc.h"
#include "version.h"
#include <cstring>

#define ENCOUNTER WalrusRPG::Undertale::BattleStates::BattleEncounter

using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;
using WalrusRPG::Undertale::battle_common_data;


ENCOUNTER::BattleEncounter(UndertaleBattleStateMachine &machine)
    : VBattleState(machine), running(false), time_counter(0), heart_x(0), heart_y(0),
      fading(false), fadeout_timer(10), build_show(false)
{
    heart_x = 160 - tex_heart->get_dimensions().width / 2;
    heart_y = 120 - tex_heart->get_dimensions().height / 2;
}

ENCOUNTER::~BattleEncounter()
{
}

void ENCOUNTER::update(unsigned dt)
{
    UNUSED(dt);

    if (Input::key_pressed(K_START) && !build_show)
        build_show = true;

    if (!fading && !running && Input::key_pressed(K_A))
    {
        fading = true;
        for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
            machine.get_enemy(i)->reset();
        machine.health = machine.health_max;
    }

    if (fading && !running)
    {
        --fadeout_timer;
        if (fadeout_timer == 0)
        {
            running = true;
        }
    }

    if (!running)
        return;
    ++time_counter;
    bool done = false;
    if (time_counter > 70 && time_counter % 3 == 2)
    {
        heart_x -= 18; // 140/8
        heart_y += 14;
        if (heart_x < 20)
        {
            done = true;
            heart_x = 20;
        }
        if (heart_y > 223)
        {
            done &= true;
            heart_y = 223;
        }
        if (done)
            machine.set_state(new BattleMainMenu(machine, true));
    }
}

void ENCOUNTER::render(unsigned dt)
{
    UNUSED(dt);
    fill(Graphics::Black);

    if (time_counter >= 50 && time_counter < 70)
    {
        if (time_counter % 8 < 4)
            put_sprite(*tex_heart, heart_x, heart_y, tex_heart->get_dimensions());
    }
    else
        put_sprite(*tex_heart, heart_x, heart_y, tex_heart->get_dimensions());

    if (!running)
    {
        unsigned factor = fadeout_timer * 255 / 10;
        Pixel mult(factor, factor, factor);
        Pixel mult_div2(factor / 2, factor / 2, factor / 2);
        put_bubble_string("Undertale battle engine demo", 2, 2, mult);
        put_hud_string("By Eiyeron", 8, 17, mult);
        put_hud_string("v", 8, 23, mult);
        put_hud_string("0.9.0", 14, 23, mult_div2);
        put_bubble_string("Proudly powered by WalrusRPG.", 2, 240 - 15, mult_div2);

        if (build_show)
        {
            put_hud_string("Build :", 8, 31, mult);
            put_hud_string(git_version, 49, 31, mult_div2);
        }


        put_bubble_string("Instructions", 0, 48, mult);
#ifdef SFML
        put_hud_string("Move  : [ Arrow keys ]", 8, 63, mult);
        put_hud_string("Enter :   [ W ]", 8, 71, mult);
        put_hud_string("Back  :   [ X ]", 8, 79, mult);
        put_hud_string("Quit  : [ Backspace ]", 8, 87, mult);
        const static char *start = "Press [W] to start!";
        const static unsigned len = strlen(start);
        put_bubble_string(start, (320 - len * 9) / 2, heart_y + 24, mult);
#elif NSPIRE
        put_hud_string("Move  : [ 8456 ]", 8, 63, mult);
        put_hud_string("Enter : [ CTRL ]", 8, 71, mult);
        put_hud_string("Back  : [ SHIFT ]", 8, 79, mult);
        put_hud_string("Quit  : [ ESC ]", 8, 87, mult);
        const static char *start = "Press [CTRL] to start!";
        const static unsigned len = strlen(start);
        put_bubble_string(start, (320 - len * 9) / 2, heart_y + 24, mult);
        put_bubble_string(start, (320 - len * 9) / 2, heart_y + 24, mult);
#endif
    }
    // // Text Frame
    // // width : 4
    // // 32/250 - 606/250
    // // |              |
    // // 32/389 - 606/389


    if (time_counter >= 20 && time_counter < 50)
        put_sprite(*tex_exclamation, 160 - tex_exclamation->get_dimensions().width / 2,
                   heart_y - tex_exclamation->get_dimensions().height,
                   tex_exclamation->get_dimensions());
}
