#include "BattleMainMenu.h"
#include "BattleEnemyAttack.h"
#include "BattleGameOver.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include <cstring>
#include <cmath>
#include "utility/minmax.h"

#define ENEMYATTACK WalrusRPG::Undertale::BattleStates::BattleEnemyAttack
using WalrusRPG::Undertale::BattleStates::BattleMainMenu;
using WalrusRPG::Undertale::BattleStates::BattleGameOver;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;
using WalrusRPG::Utils::Rect;
using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;

ENEMYATTACK::BattleEnemyAttack(UndertaleBattleStateMachine &machine)
    : VBattleState(machine), attacking(false), showing_thoughts(true), text_index(0),
      text_rot_time(0), text_box_ready(false), target_dim{119, 126, 84, 64}, heart_x(38),
      heart_y(28), timer(360), invincibility_timer(0)
{
    unsigned nb_attacks = 0;
    for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
    {
        nb_attacks += (machine.get_enemy(i)->selected_attack != nullptr);
    }
    for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
    {
        machine.get_enemy(i)->select_attack();
        machine.get_enemy(i)->selected_attack->set_combined_attack(nb_attacks > 1);
    }
}

ENEMYATTACK::~BattleEnemyAttack()
{
}

void ENEMYATTACK::update(unsigned dt)
{
    if (!text_box_ready)
    {
        Rect &tb = machine.text_box_dimensions;
        if (tb.x < target_dim.x)
            tb.x += min(target_dim.x - tb.x, 4);

        if (tb.y < target_dim.y)
            tb.y += min(target_dim.y - tb.y, 4);

        if (tb.width < target_dim.width)
            tb.width += min(target_dim.width - tb.width, 8);

        if (tb.height < target_dim.height)
            tb.height += min(target_dim.height - tb.height, 8);

        if (tb.x > target_dim.x)
            tb.x -= min(tb.x - target_dim.x, 4);

        if (tb.y > target_dim.y)
            tb.y -= min(tb.y - target_dim.y, 4);

        if (tb.width > target_dim.width)
            tb.width -= min(tb.width - target_dim.width, 8);

        if (tb.height > target_dim.height)
            tb.height -= min(tb.height - target_dim.height, 8);


        if (tb.x == target_dim.x && tb.y == target_dim.y &&
            tb.width == target_dim.width && tb.height == target_dim.height)
            text_box_ready = true;
    }
    if (showing_thoughts)
    {
        text_index += dt;
        text_rot_time += dt;
        if (Input::key_pressed(K_A))
            showing_thoughts = false;
    }

    if (attacking)
    {
        if (--timer == 0)
        {
            machine.set_state(new BattleMainMenu(machine));
            return;
        }
        for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
        {
            if (machine.get_enemy(i)->can_attack() &&
                machine.get_enemy(i)->selected_attack != nullptr)
                machine.get_enemy(i)->selected_attack->update(dt);
        }
        if (Input::key_down(K_DOWN) &&
            heart_y < machine.text_box_dimensions.height - 8 - 3) // Why -3?
            heart_y++;
        if (Input::key_down(K_UP) && heart_y > 0)
            heart_y--;
        if (Input::key_down(K_RIGHT) &&
            heart_x < machine.text_box_dimensions.width - 8 - 3)
            heart_x++;
        if (Input::key_down(K_LEFT) && heart_x > 0)
            heart_x--;

        if (invincibility_timer)
            --invincibility_timer;
        else
        {
            Rect hbox{static_cast<int>(heart_x), static_cast<int>(heart_y), 8, 8};
            for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
            {
                if (machine.get_enemy(i)->can_attack() &&
                    machine.get_enemy(i)->selected_attack->check_collision(hbox))
                {
                    invincibility_timer = 60;
                    machine.health -=
                        min(machine.get_enemy(i)
                                ->selected_attack->calculate_damage(machine.health),
                            machine.health);
                    if (machine.health == 0)
                        machine.set_state(new BattleGameOver(machine,
                                                             target_dim.x + 2 + heart_x,
                                                             target_dim.y + 2 + heart_y));
                    break;
                }
            }
        }
    }
    else if (!showing_thoughts && text_box_ready)
        attacking = true;
}

void ENEMYATTACK::render(unsigned dt)
{
    // UNUSED(dt);
    fill(Graphics::Black);
    draw_frame();
    draw_hud(machine.health, machine.health_max);
    draw_enemies(machine.get_enemies(), machine.get_nb_enemies());
    draw_text_hitbox(machine.text_box_dimensions, 2);
    if ((invincibility_timer % 4) < 2)
        put_sprite(*WalrusRPG::Undertale::tex_heart, target_dim.x + 2 + heart_x,
                   target_dim.y + 2 + heart_y, {0, 0, 8, 8});
    for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
        if (machine.get_enemy(i)->selected_attack != nullptr)
            machine.get_enemy(i)->selected_attack->render(dt);
    if (showing_thoughts)
    {
        unsigned ti = text_index >> 1;
        for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
        {
            VEnemy *current = machine.get_enemy(i);
            if (!current->can_attack())
                continue;
            const char *spoken_line = current->get_bubble();
            unsigned limit = min(strlen(spoken_line), ti);

            float angle_offset = text_rot_time / 8.;
            float speed_scale = 0.2;
            unsigned bubble_x = 6u + (current->get_cell().x + 1) * 51u;
            unsigned bubble_y = 4u + 58u * current->get_cell().y;
            put_sprite(*WalrusRPG::Undertale::tex_bubble, bubble_x, bubble_y,
                       {0, 0, 53, 59});
            for (unsigned i = 0; i < limit; i++)
            {
                char index = current->get_bubble()[i];
                put_hud_char(
                    index,
                    bubble_x + 14 + 6 * (i % 6) + 2 * cos(i * speed_scale + angle_offset),
                    bubble_y + 4 + 10 * (i / 6) + 2 * sin(i * speed_scale + angle_offset),
                    Black);
                // put_sprite(*WalrusRPG::Undertale::tex_underhudfont, {(index%16), 0, 8,
                // 8});
            }
        }
    }
}