#include "BattleFlee.h"
#include "BattleEncounter.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include "utility/misc.h"
#include "render/Animator.h"
#include "TINYSTL/vector.h"
#include <cstring>


using WalrusRPG::Undertale::BattleStates::BattleFlee;
using WalrusRPG::Undertale::BattleStates::BattleEncounter;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;
using WalrusRPG::Animator;
using WalrusRPG::Frame;
using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;


BattleFlee::BattleFlee(UndertaleBattleStateMachine &machine)
    : VBattleState(machine), text_index(0), fade_out(false), fade_timer(60), heart_x(32),
      heart_animator()
{
    tinystl::vector<Frame> heart_frames;
    heart_frames.push_back({0, 4});
    heart_frames.push_back({1, 4});
    heart_animator.add_animation(0, {heart_frames, true, 2});
}

BattleFlee::~BattleFlee()
{
}

void BattleFlee::update(unsigned dt)
{
    if (text_index < strlen("Nothing to do here!") * 2)
        text_index += dt;

    if (fade_out)
    {
        fade_timer -= dt;
        if (fade_timer == 0)
        {
            machine.set_state(new BattleEncounter(machine));
            return;
        }
    }
    else
    {
        if (heart_x > -8)
        {
            heart_x -= dt;
            heart_animator.update(dt);
        }
    }

    if (Input::key_pressed(K_A) && heart_x <= -8)
    {
        fade_out = true;
    }
}

void BattleFlee::render(unsigned dt)
{
    UNUSED(dt);
    fill(Graphics::Black);

    if (fade_out)
    {
        unsigned factor = (255 * fade_timer) / 60;
        const Pixel gray(factor, factor, factor);
        draw_frame(factor);

        draw_enemies(machine.get_enemies(), machine.get_nb_enemies(), factor);

        draw_hud(machine.health, machine.health_max, factor);

        draw_text_hitbox(machine.text_box_dimensions, 2, factor);
    }
    else
    {
        draw_frame();

        draw_enemies(machine.get_enemies(), machine.get_nb_enemies());

        draw_hud(machine.health, machine.health_max);

        draw_text_hitbox(machine.text_box_dimensions, 2);

        put_text("Nothing to do here!", "", "", text_index / 2);

        put_sprite(*tex_heart_flee, heart_x, 155,
                   {static_cast<int>(8 * heart_animator.get_animation_frame(0)), 0, 8, 12});
    }
}
