#include "BattleGameOver.h"
#include "BattleEncounter.h"
#include "GraphicsEx.h"
#include "input/Input.h"

#define GAMEOVER WalrusRPG::Undertale::BattleStates::BattleGameOver
using WalrusRPG::Undertale::BattleStates::BattleEncounter;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;
using WalrusRPG::Utils::Rect;
using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;


GAMEOVER::BattleGameOver(UndertaleBattleStateMachine &machine, unsigned heart_x,
                         unsigned heart_y)
    : VBattleState(machine), t(0), hx(heart_x), hy(heart_y), frags(), fading(false), fading_time(60)
{
	for (int i = 0; i < 4; ++i)
	{
		frags[i].pos.x = (hx+2)*8;
		frags[i].pos.y = hy*8;
		frags[i].dx = rand()%15-7;
		frags[i].dy = -(rand()%7);
		frags[i].frame = (rand()%3)*4;
	}
}

GAMEOVER::~BattleGameOver()
{
}

void GAMEOVER::update(unsigned dt)
{
    t += dt;
    // fragments
    if(t > 90)
    {
		for (int i = 0; i < 4; ++i)
		{

	    	frags[i].pos.x += frags[i].dx;
	    	frags[i].pos.y += frags[i].dy;
	    	frags[i].dy ++;
	    	frags[i].frame = (frags[i].frame+1)%(3*4);
		}
    }
    if(fading)
    {
    	if(fading_time <= dt)
	        machine.set_state(new BattleEncounter(machine));
	    else
	    	fading_time -= dt;
    }
    if (t > 240 && Input::key_pressed(K_A))
    	fading = true;
}

void GAMEOVER::render(unsigned dt)
{
    fill(Graphics::Black);
    if (t < 60)
        put_sprite(*WalrusRPG::Undertale::tex_heart, hx, hy, {0, 0, 8, 8});
    else if (t < 90)
        put_sprite(*WalrusRPG::Undertale::tex_heartbrk, hx-1, hy, {0, 0, 10, 8});
    else
    {
		for (int i = 0; i < 4; ++i)
		{

			Rect frame{static_cast<int>(frags[i].frame/4*7), 8, 7, 6};
			Graphics::put_sprite(*tex_heartbrk, frags[i].pos.x/8, frags[i].pos.y/8, frame);
		}

    }
    if(t > 120)
    {
    	if(t < 130)
    	{
    		Pixel mult(25*(t-120), 25*(t-120), 25*(t-120));
	    	Graphics::put_sprite_tint(*tex_gameover, 55, 20, {0, 0, 210, 90}, mult);
    	}
    	else if(fading)
    	{
    		unsigned factor = fading_time*(255/60.);
    		Pixel fading_mult(factor, factor, factor);
	    	Graphics::put_sprite_tint(*tex_gameover, 55, 20, {0, 0, 210, 90}, fading_mult);
    	}
    	else
	    	Graphics::put_sprite(*tex_gameover, 55, 20, {0, 0, 210, 90});

	    if(t >  150)
	    {
	    	if(fading)
	    	{
	    		unsigned factor = fading_time*(255/60.);
	    		Pixel fading_mult(factor, factor, factor);
		    	Graphics::put_string_clip("Hey, try again, will you?", 60, 160, (t-150)/8, fading_mult);
	    	}
	    	else
		    	Graphics::put_string_clip("Hey, try again, will you?", 60, 160, (t-150)/8);
	    }
    }
}
