#include "BattleItemSelection.h"
#include "BattleMainMenu.h"
#include "UndertaleBattleStateMachine.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include "utility/misc.h"

using WalrusRPG::Undertale::BattleStates::BattleItemSelection;
using WalrusRPG::Undertale::BattleStates::BattleMainMenu;

using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;

BattleItemSelection::BattleItemSelection(UndertaleBattleStateMachine &machine)
    : VBattleState(machine), cursor(0)
{
}

BattleItemSelection::~BattleItemSelection()
{
}

void BattleItemSelection::update(unsigned dt)
{
    UNUSED(dt);
    if (machine.get_nb_items() > 0)
    {
        if (Input::key_pressed(K_DOWN))
            cursor = (cursor + 2) % machine.get_nb_items();
        if (Input::key_pressed(K_UP))
            cursor = (cursor - 2) % machine.get_nb_items();
        if (Input::key_pressed(K_LEFT))
            cursor = (cursor - 1) % machine.get_nb_items();
        if (Input::key_pressed(K_RIGHT))
            cursor = (cursor + 1) % machine.get_nb_items();
    }

    if (Input::key_pressed(K_A) && cursor < machine.get_nb_items())
        machine.get_item(cursor)->use(machine);

    if (Input::key_pressed(K_B))
        machine.set_state(new BattleMainMenu(machine));
}

void BattleItemSelection::render(unsigned dt)
{
    UNUSED(dt);
    fill(Graphics::Black);

    draw_frame();

    draw_enemies(machine.get_enemies(), machine.get_nb_enemies());

    draw_hud(machine.health, machine.health_max);

    draw_text_hitbox(machine.text_box_dimensions, 2);

    draw_menu_buttons(2);

    put_sprite(*tex_heart, 32 + 128 * ((cursor % 5) % 2),
               135 + 4 + 16 * ((cursor % 5) / 2), tex_heart->get_dimensions());


    if (machine.get_items() == nullptr)
        return;

    if (machine.get_nb_items() > 0)
    {
        unsigned item_list_start = (cursor / 5) * 5;
        unsigned item_list_stop = item_list_start + 5;
        if (item_list_stop > machine.get_nb_items())
            item_list_stop = machine.get_nb_items();
        for (unsigned i = 0; i < item_list_stop - item_list_start; i++)
        {
            put_string("*", 51 + 128 * (i % 2), 135 + (i / 2) * 16);
            put_string(machine.get_item(item_list_start + i)->get_short_name(false),
                       67 + 128 * (i % 2), 135 + (i / 2) * 16);
        }
        if (machine.get_nb_items() > 5)
        {
            char numbers[] = "1234567890";
            put_string("PAGE", 179, 167);
            put_char(numbers[cursor / 5], 219, 167);
        }
    }
}
