#include "BattleMainMenu.h"
#include "BattleActTargetSelection.h"
#include "BattleMercySelection.h"
#include "BattleAttackTargetSelection.h"
#include "BattleTextTransition.h"
#include "BattleItemSelection.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include "utility/Rect.h"
#include "utility/misc.h"
#include "utility/minmax.h"
#include "Item.h"

#define MAINMENU WalrusRPG::Undertale::BattleStates::BattleMainMenu

using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;
using WalrusRPG::Undertale::IdleText;
using WalrusRPG::Undertale::Item;
using WalrusRPG::Undertale::BattleStates::BattleTextTransition;
using WalrusRPG::Undertale::BattleStates::BattleActTargetSelection;
using WalrusRPG::Undertale::BattleStates::BattleAttackTargetSelection;
using WalrusRPG::Undertale::BattleStates::BattleItemSelection;
using WalrusRPG::Undertale::BattleStates::BattleMercySelection;
using WalrusRPG::Utils::Rect;


namespace
{
    static const unsigned fade_in_max = 10;
    static const Rect target_dim{16, 125, 287, 70};
}

MAINMENU::BattleMainMenu(UndertaleBattleStateMachine &parent, bool fade_in)
    : VBattleState(parent), fade_in(fade_in), main_menu_position(0), text_index(0),
      text_box_ready(false)
{
    // parent.
    fade_in_timer = (fade_in ? fade_in_max : 0);
}

MAINMENU::BattleMainMenu(UndertaleBattleStateMachine &parent)
    : BattleMainMenu(parent, false)
{
}

MAINMENU::~BattleMainMenu()
{
}

void MAINMENU::update(unsigned dt)
{
    UNUSED(dt);
    if (fade_in)
    {
        --fade_in_timer;
        if (fade_in_timer == 0)
            fade_in = false;
        return;
    }
    if (!text_box_ready)
    {
        Rect &tb = machine.text_box_dimensions;
        if (tb.x < target_dim.x)
            tb.x += min(target_dim.x - tb.x, 4);

        if (tb.y < target_dim.y)
            tb.y += min(target_dim.y - tb.y, 4);

        if (tb.width < target_dim.width)
            tb.width += min(target_dim.width - tb.width, 8);

        if (tb.height < target_dim.height)
            tb.height += min(target_dim.height - tb.height, 8);

        if (tb.x > target_dim.x)
            tb.x -= min(tb.x - target_dim.x, 4);

        if (tb.y > target_dim.y)
            tb.y -= min(tb.y - target_dim.y, 4);

        if (tb.width > target_dim.width)
            tb.width -= min(tb.width - target_dim.width, 8);

        if (tb.height > target_dim.height)
            tb.height -= min(tb.height - target_dim.height, 8);


        if (tb.x == target_dim.x && tb.y == target_dim.y &&
            tb.width == target_dim.width && tb.height == target_dim.height)
            text_box_ready = true;
        return;
    }
    ++text_index;
    if (Input::key_pressed(K_LEFT) && main_menu_position > 0)
        main_menu_position--;
    if (Input::key_pressed(K_RIGHT) && main_menu_position < 3)
        main_menu_position++;

    if (Input::key_pressed(K_A))
    {
        switch (main_menu_position)
        {
            case 0:
                // battle_menu_state = BMS_ATTACK_ENEMY_SELECTION;
                machine.set_state(new BattleAttackTargetSelection(
                    machine, new BattleMainMenu(machine)));
                break;
            case 1:
                machine.set_state(new BattleActTargetSelection(machine));
                break;
            case 2:
                machine.set_state(new BattleItemSelection(machine));
                break;
            case 3:
                machine.set_state(new BattleMercySelection(machine, true));
                break;
            default:
                // main_menu_position = 0;
                break;
        }
    }
}

void MAINMENU::render(unsigned dt)
{
    UNUSED(dt);
    unsigned fade_in_scale =
        fade_in_timer ? 255 * (fade_in_max - fade_in_timer) / fade_in_max : 255;
    Pixel fade_in_multiplier(fade_in_scale, fade_in_scale, fade_in_scale);
    // /2 rounded
    fill(Graphics::Black);

    draw_frame(fade_in_scale);

    if (fade_in)
    {
        draw_enemies(machine.get_enemies(), machine.get_nb_enemies(), fade_in_scale);
        draw_menu_buttons(main_menu_position, fade_in_scale);
        draw_hud(machine.health, machine.health_max, fade_in_scale);
    }
    else
    {
        draw_enemies(machine.get_enemies(), machine.get_nb_enemies());
        draw_menu_buttons(main_menu_position);
        draw_hud(machine.health, machine.health_max);
    }


    // // Text Frame
    // // width : 4
    // // 32/250 - 606/250
    // // |              |
    // // 32/389 - 606/389
    draw_text_hitbox(machine.text_box_dimensions, 2, fade_in_scale);

    unsigned heart_x_positions[] = {20, 97, 177, 254};
    put_sprite(*tex_heart, heart_x_positions[main_menu_position], 223,
               tex_heart->get_dimensions());


    // TODO : fix this shit.
    if (machine.get_enemy(machine.selected_enemy_index) != nullptr)
    {
        IdleText &lines = machine.get_enemy(machine.selected_enemy_index)->get_lines();
        put_text(lines.line1, lines.line2, lines.line3, text_index / 2);
    }
    else
        put_text("* This is a proof-of-concept test.", "I don't want to remake the game.",
                 "Just wanting to see if could work.", text_index / 2);
}
