#include "BattleMercySelection.h"
#include "BattleEnemyAttack.h"
#include "BattleMainMenu.h"
#include "BattleFlee.h"
#include "BattleVictory.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include "utility/misc.h"

using WalrusRPG::Undertale::BattleStates::BattleMercySelection;
using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;
using WalrusRPG::Undertale::BattleStates::BattleMainMenu;
using WalrusRPG::Undertale::BattleStates::BattleEnemyAttack;
using WalrusRPG::Undertale::BattleStates::BattleFlee;
using WalrusRPG::Undertale::BattleStates::BattleVictory;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;


BattleMercySelection::BattleMercySelection(UndertaleBattleStateMachine &machine,
                                           bool can_flee)
    : VBattleState(machine), cursor(0), can_flee(can_flee)
{
}

BattleMercySelection::~BattleMercySelection()
{
}

void BattleMercySelection::update(unsigned dt)
{
    UNUSED(dt);
    if (can_flee && (Input::key_pressed(K_DOWN) || Input::key_pressed(K_UP)))
    {
        cursor = (cursor + 1) % 2;
    }
    if (Input::key_pressed(K_B))
        machine.set_state(new BattleMainMenu(machine));
    if (Input::key_pressed(K_A))
    {
        if (cursor == 0)
        {
            bool all_spared = true;
            for (unsigned i = 0; i < machine.get_nb_enemies(); i++)
            {
                machine.get_enemy(i)->on_spare();
                if (!machine.get_enemy(i)->is_spared())
                    all_spared = false;
            }
            if (all_spared)
                machine.set_state(new BattleVictory(machine));
            else
                machine.set_state(new BattleEnemyAttack(machine));
        }
        else
        {
            machine.set_state(new BattleFlee(machine));
        }
    }
}

void BattleMercySelection::render(unsigned dt)
{
    UNUSED(dt);

    fill(Graphics::Black);

    draw_frame();

    draw_enemies(machine.get_enemies(), machine.get_nb_enemies());

    draw_hud(machine.health, machine.health_max);

    draw_text_hitbox(machine.text_box_dimensions, 2);

    put_string("* Spare", 51, 135);
    if (can_flee)
        put_string("* Flee", 51, 151);

    draw_menu_buttons(4);
    put_sprite(*tex_heart, 32, 135 + 4 + 16 * cursor, tex_heart->get_dimensions());
}
