#include "BattleVictory.h"
#include "BattleEncounter.h"
#include "GraphicsEx.h"
#include "input/Input.h"
#include "utility/misc.h"
#include "render/Animator.h"
#include "TINYSTL/vector.h"
#include <cstring>
#include <cstdio>

using WalrusRPG::Undertale::BattleStates::BattleVictory;
using WalrusRPG::Undertale::BattleStates::BattleEncounter;
using WalrusRPG::Undertale::UndertaleBattleStateMachine;
using WalrusRPG::Animator;
using WalrusRPG::Frame;
using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;


BattleVictory::BattleVictory(UndertaleBattleStateMachine &machine)
    : VBattleState(machine), text_index(0), text_finished(false), fading(false),
      fade_timer(60), text(""), text2("")
{
    unsigned xp_won = 0;
    unsigned coins_won = 90;
    sprintf(text, "* You won %u EXP", xp_won);
    sprintf(text2, "and %u coins!", coins_won);
}

BattleVictory::~BattleVictory()
{
}

void BattleVictory::update(unsigned dt)
{
    UNUSED(dt);
    if (text_index < (strlen("* Victory!") + strlen(text) + strlen(text2)) * 2)
        text_index++;
    else if (!text_finished)
    {
        text_finished = true;
    }

    if (fading)
    {
        fade_timer--;
        if (fade_timer == 0)
        {
            machine.set_state(new BattleEncounter(machine));
            return;
        }
    }
    else
    {
    }

    if (Input::key_pressed(K_A) && text_finished)
    {
        fading = true;
    }
}

void BattleVictory::render(unsigned dt)
{
    UNUSED(dt);
    fill(Graphics::Black);

    if (fading)
    {
        unsigned factor = (255 * fade_timer) / 60;
        const Pixel gray(factor, factor, factor);
        draw_frame(factor);

        draw_enemies(machine.get_enemies(), machine.get_nb_enemies(), factor);

        draw_hud(machine.health, machine.health_max, factor);

        draw_text_hitbox(machine.text_box_dimensions, 2, factor);
    }
    else
    {
        draw_frame();

        draw_enemies(machine.get_enemies(), machine.get_nb_enemies());

        draw_hud(machine.health, machine.health_max);

        draw_text_hitbox(machine.text_box_dimensions, 2);

        put_text("* Victory!", text, text2, text_index / 2);
    }
}
