#include "GraphicsEx.h"
#include "utility/Rect.h"
#include "utility/misc.h"
#include <cstring>
#include <cmath>

#define GRAPHICS WalrusRPG::Graphics
using WalrusRPG::Graphics::Texture;
using WalrusRPG::Graphics::Pixel;
using WalrusRPG::Utils::Rect;
using WalrusRPG::Undertale::VEnemy;
using namespace WalrusRPG::Undertale;


void GRAPHICS::put_char(unsigned char c, unsigned x, unsigned y, const Pixel &color)
{
    signed c_x = 8 * (c % 16);
    signed c_y = 16 * (c / 16);
    Graphics::put_sprite_tint(*tex_underfont, x, y, Rect(c_x, c_y, 8, 16), color);
}

void GRAPHICS::put_char(unsigned char c, unsigned x, unsigned y)
{
    signed c_x = 8 * (c % 16);
    signed c_y = 16 * (c / 16);
    Graphics::put_sprite(*tex_underfont, x, y, Rect(c_x, c_y, 8, 16));
}

void GRAPHICS::put_hud_char(unsigned char c, unsigned x, unsigned y)
{
    unsigned c_x = 5 * (c % 16);
    unsigned c_y = 6 * (c / 16);
    Graphics::put_sprite(*tex_underhudfont, x, y, Rect(c_x, c_y, 5, 6));
}

void GRAPHICS::put_hud_char(unsigned char c, unsigned x, unsigned y, const Pixel &color)
{
    unsigned c_x = 5 * (c % 16);
    unsigned c_y = 6 * (c / 16);
    Graphics::put_sprite_tint(*tex_underhudfont, x, y, Rect(c_x, c_y, 5, 6), color);
}

void GRAPHICS::put_bubble_char(unsigned char c, unsigned x, unsigned y)
{
    unsigned c_x = 9 * (c % 16);
    unsigned c_y = 14 * (c / 16);
    Graphics::put_sprite(*tex_bubblefont, x, y, Rect(c_x, c_y, 9, 14));
}

void GRAPHICS::put_bubble_char(unsigned char c, unsigned x, unsigned y,
                               const Pixel &color)
{
    unsigned c_x = 9 * (c % 16);
    unsigned c_y = 14 * (c / 16);
    Graphics::put_sprite_tint(*tex_bubblefont, x, y, Rect(c_x, c_y, 9, 14), color);
}

void GRAPHICS::put_string_clip(const char *str, unsigned x, unsigned y, unsigned max_length,
                     const Pixel &color)
{
    for (unsigned i = 0; str[i] && i < max_length; ++i)
    {
        signed off_x = 0, off_y = 0;
        if (rand() % 500 == 0)
        {
            off_x = rand() % 3 - 1;
            off_y = rand() % 3 - 1;
        }
        GRAPHICS::put_char(str[i], x + 8 * i + off_x, y + off_y, color);
    }
}
void GRAPHICS::put_string_clip(const char *str, unsigned x, unsigned y, unsigned max_length)
{
    for (unsigned i = 0; str[i] && i < max_length; ++i)
    {
        signed off_x = 0, off_y = 0;
        if (rand() % 500 == 0)
        {
            off_x = rand() % 3 - 1;
            off_y = rand() % 3 - 1;
        }
        GRAPHICS::put_char(str[i], x + 8 * i + off_x, y + off_y);
    }
}

void GRAPHICS::put_text(const tinystl::string &line1, const tinystl::string &line2,
                        const tinystl::string &line3, unsigned text_index)
{
    const char *l1 = line1.c_str();
    put_string_clip(l1, 24, 136, text_index);
    if (text_index > line1.size())
    {
        const char *l2 = line2.c_str();
        put_string_clip(l2, 24, 152, text_index - line1.size());
        if (text_index > line1.size() + line2.size())
        {
            const char *l3 = line3.c_str();
            put_string_clip(l3, 24, 168, text_index - line1.size() - line2.size());
        }
    }
}

void GRAPHICS::put_string(const tinystl::string &str, unsigned x, unsigned y)
{
    put_string_clip(str.c_str(), x, y, strlen(str.c_str()));
}

void GRAPHICS::put_string(const tinystl::string &str, unsigned x, unsigned y,
                          const Pixel &color)
{
    put_string_clip(str.c_str(), x, y, strlen(str.c_str()), color);
}

void GRAPHICS::put_hud_string(const tinystl::string &str, unsigned x, unsigned y)
{
    const char *str_c = str.c_str();
    for (unsigned i = 0; str_c[i]; ++i)
        Graphics::put_hud_char(str_c[i], x + i * 6, y);
}

void GRAPHICS::put_hud_string(const tinystl::string &str, unsigned x, unsigned y,
                              const Pixel &color)
{
    const char *str_c = str.c_str();
    for (unsigned i = 0; str_c[i]; ++i)
        Graphics::put_hud_char(str_c[i], x + i * 6, y, color);
}

void GRAPHICS::put_bubble_string(const tinystl::string &str, unsigned x, unsigned y)
{
    const char *str_c = str.c_str();
    for (unsigned i = 0; str_c[i]; ++i)
        Graphics::put_bubble_char(str_c[i], x + i * 9, y);
}

void GRAPHICS::put_bubble_string(const tinystl::string &str, unsigned x, unsigned y,
                                 const Pixel &color)
{
    const char *str_c = str.c_str();
    for (unsigned i = 0; str_c[i]; ++i)
        Graphics::put_bubble_char(str_c[i], x + i * 9, y, color);
}

void GRAPHICS::draw_frame(unsigned fade_in_color)
{
    Pixel grid_color(34 * fade_in_color / 255, 177 * fade_in_color / 255,
                     76 * fade_in_color / 255);
    // Green frame
    for (unsigned i = 0; i < 7; i++)
        put_vertical_line(6 + 51 * i, 4, 120, grid_color);

    for (unsigned i = 0; i < 3; i++)
        put_horizontal_line(6, 312, 4 + 58 * i, grid_color);
}

void GRAPHICS::draw_frame()
{
    draw_frame(255);
}

void GRAPHICS::draw_text_hitbox(const WalrusRPG::Utils::Rect &dim, unsigned stroke_width)
{
    draw_text_hitbox(dim, stroke_width, 255);
}


void GRAPHICS::draw_text_hitbox(const WalrusRPG::Utils::Rect &dim, unsigned stroke_width,
                                unsigned fade_in_color)
{
    Pixel white(fade_in_color, fade_in_color, fade_in_color);
    for (unsigned i = 0; i < stroke_width; i++)
    {
        put_vertical_line(dim.x + i, dim.y, dim.y + dim.height, white);
        put_vertical_line(dim.x + dim.width - i, dim.y, dim.y + dim.height, white);
        put_horizontal_line(dim.x, dim.x + dim.width, dim.y + i, white);
        put_horizontal_line(dim.x, dim.x + dim.width, dim.y + dim.height - i, white);
    }
}

void GRAPHICS::draw_enemy_damage_missed(signed y_offset,
                                        const WalrusRPG::Utils::Rect &cell)
{
    Rect center({6 + 26 + 51 * cell.x, 4 + 58 * cell.y + 29});
    Rect pos({center.x - 29, center.y - 64 + y_offset});
    put_sprite_tint(*tex_miss, pos.x, pos.y, {0, 0, 58, 14}, Red);
}

void GRAPHICS::draw_enemy_damage(unsigned damage_done, signed y_offset,
                                 const WalrusRPG::Utils::Rect &cell)
{
    unsigned nb_digits = (damage_done == 0 ? 1 : log10(damage_done) + 1);
    Rect center({6 + 26 + 51 * cell.x, 4 + 58 * cell.y + 29});
    Rect dmg_begin({static_cast<int>(center.x + 8 * nb_digits - 16), center.y - 64 + y_offset});
    for (unsigned i = 0; i < nb_digits; i++)
    {
        unsigned current_digit = damage_done % 10;
        put_sprite_tint(*tex_dmgfont, dmg_begin.x, dmg_begin.y,
                        {static_cast<int>(16 * current_digit), 0, 16, 16}, Red);
        dmg_begin.x -= 16;
        damage_done /= 10;
    }
}


// TODO : Probably use a better rule for enemy bigger, on the top row or special ones?
void GRAPHICS::draw_enemy_life_bar(unsigned hp_max, unsigned hp, const Rect &cell)
{
    Rect center({6 + 26 + 51 * cell.x, 4 + 58 * cell.y + 29});
    put_rectangle({center.x - 20, center.y - 45, 40, 10}, Red);
    put_rectangle({center.x - 20, center.y - 45, ((40 * hp) / hp_max), 10},
                  Green);
}

void GRAPHICS::draw_enemies(VEnemy **enemies, unsigned nb_enemies)
{
    for (unsigned i = 0; i < nb_enemies; i++)
    {
        Rect r({6 + 26 + 51 * enemies[i]->get_cell().x,
                4 + 58 * enemies[i]->get_cell().y + 29});
        enemies[i]->render(r);
    }
}
void GRAPHICS::draw_enemies(VEnemy **enemies, unsigned nb_enemies, unsigned brightness)
{
    const Pixel gray(brightness, brightness, brightness);
    for (unsigned i = 0; i < nb_enemies; i++)
    {
        Rect r({6 + 26 + 51 * enemies[i]->get_cell().x,
                4 + 58 * enemies[i]->get_cell().y + 29});
        enemies[i]->render(r, gray);
    }
}

void GRAPHICS::draw_menu_buttons(unsigned selected_button)
{
    put_sprite(*tex_fight, 16, 216, {0, 21 * (selected_button == 0 ? 1 : 0), 55, 21});
    put_sprite(*tex_act, 93, 216, {0, 21 * (selected_button == 1 ? 1 : 0), 55, 21});
    put_sprite(*tex_item, 173, 216, {0, 21 * (selected_button == 2 ? 1 : 0), 55, 21});
    put_sprite(*tex_mercy, 250, 216, {0, 21 * (selected_button == 3 ? 1 : 0), 55, 21});
}
void GRAPHICS::draw_menu_buttons(unsigned selected_button, unsigned brightness)
{
    const Pixel gray(brightness, brightness, brightness);
    put_sprite_tint(*tex_fight, 16, 216,
                    {0, 21 * (selected_button == 0 ? 1 : 0), 55, 21}, gray);
    put_sprite_tint(*tex_act, 93, 216, {0, 21 * (selected_button == 1 ? 1 : 0), 55, 21},
                    gray);
    put_sprite_tint(*tex_item, 173, 216,
                    {0, 21 * (selected_button == 2 ? 1 : 0), 55, 21}, gray);
    put_sprite_tint(*tex_mercy, 250, 216,
                    {0, 21 * (selected_button == 3 ? 1 : 0), 55, 21}, gray);
}


void GRAPHICS::draw_hud(unsigned hp, unsigned hp_max)
{
    put_hud_string("Eiyeron", 15, 203);

    put_rectangle({137, 200, (hp_max * 5) / 8, 10}, Red);
    put_rectangle({137, 200, (hp * 5) / 8, 10}, Yellow);
    char buffer[16] = {0};
    sprintf(buffer, "HP %02u/%02u", hp, hp_max);
    put_hud_string(buffer, 137 + int(hp_max * 5) / 8 + 6, 203);
}
void GRAPHICS::draw_hud(unsigned hp, unsigned hp_max, unsigned brightness)
{
    const Pixel gray(brightness, brightness, brightness);
    const Pixel red(brightness, 0, 0);
    const Pixel yellow(brightness, brightness, 0);
    put_hud_string("Eiyeron", 15, 203, gray);

    put_rectangle({137, 200, (hp_max * 5) / 8, 10}, red);
    put_rectangle({137, 200, (hp * 5) / 8, 10}, yellow);
    char buffer[16] = {0};
    sprintf(buffer, "HP %02u/%02u", hp, hp_max);
    put_hud_string(buffer, 137 + (hp_max * 5) / 8 + 6, 203, gray);
}
