#include "UndertaleBattleStateMachine.h"
#include "Graphics.h"
#include "enemies/Froggit.h"
#include "piaf/Archive.h"

#include "items/Stick.h"
#include "items/MonsterCandy.h"

#define BATTLESTATEMACHINE WalrusRPG::Undertale::UndertaleBattleStateMachine

using WalrusRPG::States::StateUndertaleBattle;
using WalrusRPG::Undertale::BattleStates::VBattleState;
using WalrusRPG::Undertale::VEnemy;
using WalrusRPG::Undertale::Item;
using WalrusRPG::Undertale::Enemies::Froggit;

using WalrusRPG::Undertale::Items::Stick;
using WalrusRPG::Undertale::Items::MonsterCandy;

using WalrusRPG::Graphics::Texture;
using WalrusRPG::PIAF::Archive;
using WalrusRPG::PIAF::File;

Archive *WalrusRPG::Undertale::battle_common_data;
Texture *WalrusRPG::Undertale::tex_heart;
Texture *WalrusRPG::Undertale::tex_heartbrk;
Texture *WalrusRPG::Undertale::tex_exclamation;
Texture *WalrusRPG::Undertale::tex_fight;
Texture *WalrusRPG::Undertale::tex_act;
Texture *WalrusRPG::Undertale::tex_item;
Texture *WalrusRPG::Undertale::tex_mercy;
Texture *WalrusRPG::Undertale::tex_heart_flee;
Texture *WalrusRPG::Undertale::tex_underfont;
Texture *WalrusRPG::Undertale::tex_underhudfont;
Texture *WalrusRPG::Undertale::tex_bubblefont;
Texture *WalrusRPG::Undertale::tex_dmgfont;
Texture *WalrusRPG::Undertale::tex_aim;
Texture *WalrusRPG::Undertale::tex_miss;
Texture *WalrusRPG::Undertale::tex_bubble;
Texture *WalrusRPG::Undertale::tex_gameover;

Texture *WalrusRPG::Undertale::tex_froggit;
Texture *WalrusRPG::Undertale::tex_froggit_bullet;

BATTLESTATEMACHINE::UndertaleBattleStateMachine(StateUndertaleBattle &parent)
    : parent(parent), current_state(nullptr), enemies(nullptr), nb_enemies(0),
      items(nullptr), nb_items(0), selected_enemy_index(0), health(20), health_max(20),
      text_box_dimensions{16, 125, 287, 70}, parts()
{
    WalrusRPG::Undertale::battle_common_data = new Archive("data/battle.wrf");
    WalrusRPG::Undertale::tex_heart = new Texture(battle_common_data->get("heart"));
    WalrusRPG::Undertale::tex_heartbrk = new Texture(battle_common_data->get("heartbrk"));
    WalrusRPG::Undertale::tex_exclamation = new Texture(battle_common_data->get("exlam"));
    WalrusRPG::Undertale::tex_fight = new Texture(battle_common_data->get("fight"));
    WalrusRPG::Undertale::tex_act = new Texture(battle_common_data->get("act"));
    WalrusRPG::Undertale::tex_item = new Texture(battle_common_data->get("item"));
    WalrusRPG::Undertale::tex_mercy = new Texture(battle_common_data->get("mercy"));
    WalrusRPG::Undertale::tex_heart_flee =
        new Texture(battle_common_data->get("heart_flee"));
    WalrusRPG::Undertale::tex_froggit = new Texture(battle_common_data->get("froggit"));
    WalrusRPG::Undertale::tex_froggit_bullet =
        new Texture(battle_common_data->get("frgtbult"));
    WalrusRPG::Undertale::tex_underfont = new Texture(battle_common_data->get("ufont"));
    WalrusRPG::Undertale::tex_bubblefont =
        new Texture(battle_common_data->get("bublfont"));
    WalrusRPG::Undertale::tex_underhudfont =
        new Texture(battle_common_data->get("uhudfont"));
    WalrusRPG::Undertale::tex_dmgfont = new Texture(battle_common_data->get("dmgfont"));
    WalrusRPG::Undertale::tex_miss = new Texture(battle_common_data->get("miss"));
    WalrusRPG::Undertale::tex_bubble = new Texture(battle_common_data->get("bubble"));
    WalrusRPG::Undertale::tex_aim = new Texture(battle_common_data->get("aim"));
    WalrusRPG::Undertale::tex_gameover = new Texture(battle_common_data->get("gameover"));

    enemies = new VEnemy *[3];
    for (unsigned x = 0; x < 3; ++x)
    {
        enemies[x] = new Froggit(*this, x * 2, 1);
    }
    nb_enemies = 3;
    items = new Item *[10];
    items[0] = new Stick();
    items[1] = new MonsterCandy();
    items[2] = new MonsterCandy();
    nb_items = 3;
}

BATTLESTATEMACHINE::~UndertaleBattleStateMachine()
{
    if (current_state != nullptr)
        delete current_state;
    delete[] enemies;
    for (unsigned i = 0; i < nb_items; i++)
        delete items[i];
    delete[] items;
    delete WalrusRPG::Undertale::battle_common_data;
}

void BATTLESTATEMACHINE::set_state(VBattleState *new_state)
{
    delete current_state;
    current_state = new_state;
}


StateUndertaleBattle &BATTLESTATEMACHINE::get_parent()
{
    return parent;
}

void BATTLESTATEMACHINE::update(unsigned dt)
{
    if (current_state != nullptr)
        current_state->update(dt);
    for (unsigned i = 0; i < nb_enemies; i++)
        enemies[i]->update(dt);
    if(!parts.empty())
    {
        for(auto p = parts.begin(); p < parts.end(); p++){
            p->update(dt);
            if(!p->alive)
                parts.erase(p);
        }
    }
}

void BATTLESTATEMACHINE::render(unsigned dt)
{
    if (current_state != nullptr)
        current_state->render(dt);
    if(!parts.empty())
    {
        for(auto p = parts.begin(); p < parts.end(); p++)
            p->render(dt);
    }
}

VEnemy **BATTLESTATEMACHINE::get_enemies()
{
    return enemies;
}
VEnemy *BATTLESTATEMACHINE::get_enemy(unsigned index)
{
    if (index < nb_enemies)
        return enemies[index];
    return nullptr;
}

unsigned BATTLESTATEMACHINE::get_nb_enemies()
{
    return nb_enemies;
}

Item **BATTLESTATEMACHINE::get_items()
{
    return items;
}

Item *BATTLESTATEMACHINE::get_item(unsigned index)
{
    if (index > nb_items)
        return nullptr;
    return items[index];
}

void BATTLESTATEMACHINE::drop_item(unsigned index)
{
    if (index > nb_items)
        return;
    delete items[index];
    for (unsigned i = index + 1; i < nb_items; i++)
        items[i - 1] = items[i];
    items[nb_items - 1] = nullptr;
    nb_items--;
}

void BATTLESTATEMACHINE::drop_item(Item *instance)
{
    for (unsigned i = 0; i < nb_items; i++)
    {
        if (items[i] == instance)
            drop_item(i);
    }
}

unsigned BATTLESTATEMACHINE::get_nb_items()
{
    return nb_items;
}
