#ifndef INCLUDE_UNDERTALEBATTLESTATEMACHINE_H
#define INCLUDE_UNDERTALEBATTLESTATEMACHINE_H

#include "TINYSTL/vector.h"
#include "Particle.h"
#include "Texture.h"
#include "piaf/Archive.h"
#include "VBattleState.h"

namespace WalrusRPG
{
    namespace States
    {
        class StateUndertaleBattle;
    }
}

namespace WalrusRPG
{
    namespace Undertale
    {
        extern WalrusRPG::PIAF::Archive *battle_common_data;
        extern WalrusRPG::Graphics::Texture *tex_heart;
        extern WalrusRPG::Graphics::Texture *tex_heartbrk;
        extern WalrusRPG::Graphics::Texture *tex_heart_flee;
        extern WalrusRPG::Graphics::Texture *tex_exclamation;
        extern WalrusRPG::Graphics::Texture *tex_fight;
        extern WalrusRPG::Graphics::Texture *tex_act;
        extern WalrusRPG::Graphics::Texture *tex_item;
        extern WalrusRPG::Graphics::Texture *tex_mercy;
        extern WalrusRPG::Graphics::Texture *tex_underfont;
        extern WalrusRPG::Graphics::Texture *tex_underhudfont;
        extern WalrusRPG::Graphics::Texture *tex_bubblefont;
        extern WalrusRPG::Graphics::Texture *tex_dmgfont;
        extern WalrusRPG::Graphics::Texture *tex_aim;
        extern WalrusRPG::Graphics::Texture *tex_miss;
        extern WalrusRPG::Graphics::Texture *tex_bubble;
        extern WalrusRPG::Graphics::Texture *tex_gameover;

        extern WalrusRPG::Graphics::Texture *tex_froggit;
        extern WalrusRPG::Graphics::Texture *tex_froggit_bullet;

        class VEnemy;
        class Item;
        class UndertaleBattleStateMachine
        {
          protected:
            WalrusRPG::States::StateUndertaleBattle &parent;
            WalrusRPG::Undertale::BattleStates::VBattleState *current_state;
            VEnemy **enemies;
            unsigned nb_enemies;
            Item **items;
            unsigned nb_items;


          public:
            unsigned health;
            unsigned health_max;
            unsigned selected_enemy_index;
            tinystl::vector<Particle> parts;
            WalrusRPG::Utils::Rect text_box_dimensions;

            UndertaleBattleStateMachine(WalrusRPG::States::StateUndertaleBattle &parent);
            ~UndertaleBattleStateMachine();
            void set_state(WalrusRPG::Undertale::BattleStates::VBattleState *new_state);
            WalrusRPG::States::StateUndertaleBattle &get_parent();

            void update(unsigned dt);
            void render(unsigned dt);
            VEnemy **get_enemies();
            VEnemy *get_enemy(unsigned index);
            unsigned get_nb_enemies();
            Item **get_items();
            Item *get_item(unsigned index);
            unsigned get_nb_items();

            void drop_item(unsigned index);
            void drop_item(Item *instance);
        };
    }
}

#include "StateUndertaleBattle.h"
#include "VEnemy.h"
#include "Item.h"
#endif
