#ifndef INCLUDE_VENEMY_H
#define INCLUDE_VENEMY_H

#include "Graphics.h"
#include "utility/Rect.h"
#include "UndertaleBattleStateMachine.h"
#include "DoableActs.h"
#include "VEnemyAttack.h"

namespace WalrusRPG
{
    namespace Undertale
    {
        struct IdleText
        {
            char const *line1;
            char const *line2;
            char const *line3;
        };

        class Act;
        class Item;

        class VEnemy
        {
          protected:
            UndertaleBattleStateMachine &machine;
            WalrusRPG::Graphics::Texture &tex;
            const char *name;
            bool show_lifebar_in_menu;
            bool spareable;
            bool spared;
            unsigned hp;
            unsigned hp_max;
            unsigned att;
            unsigned def;
            unsigned bonus_att; // Lasts one turn.
            unsigned bonus_def;
            signed cell_x;
            signed cell_y;

          public:
            VEnemyAttack *selected_attack;
            VEnemy(WalrusRPG::Graphics::Texture &tex, char const *name,
                   bool show_lifebar_in_menu, UndertaleBattleStateMachine &machine,
                   unsigned hp, unsigned att, unsigned def, unsigned cell_x,
                   unsigned cell_y);
            virtual ~VEnemy();

            virtual IdleText &get_lines() = 0;
            virtual const char *get_bubble() = 0;
            virtual void on_act(Act &act) = 0;
            virtual void on_attack(unsigned damage) = 0;
            virtual void on_item(Item &item) = 0;
            virtual void on_spare() = 0;

            virtual void reset();

            unsigned get_hp();
            unsigned get_hp_max();
            unsigned get_attack();
            unsigned get_defense();

            bool is_spareable();
            bool is_spared();
            bool is_dead();
            bool can_attack();
            bool should_show_lifebar_in_menu();
            const char *get_name();

            virtual void select_attack() = 0;
            VEnemyAttack *get_selected_attack();

            WalrusRPG::Graphics::Texture &get_texture();
            WalrusRPG::Utils::Rect get_cell();

            virtual void update(unsigned dt) = 0;
            virtual void render(WalrusRPG::Utils::Rect &center) = 0;
            virtual void render(WalrusRPG::Utils::Rect &center,
                                const WalrusRPG::Graphics::Pixel &tint) = 0;

            virtual doable_acts_e *get_doable_acts() = 0;
        };
    }
}

#include "Act.h"
#include "Item.h"

#endif
