#include "FroggitFlyAttack.h"
#include "input/Input.h"
#include "GraphicsEx.h"
#include "UndertaleBattleStateMachine.h"
#include <cstdlib>

#define FROGGITATTACK WalrusRPG::Undertale::EnemyAttacks::FroggitFlyAttack
using namespace WalrusRPG::Graphics;
using namespace WalrusRPG::Input;
using WalrusRPG::Utils::Rect;

FROGGITATTACK::FroggitFlyAttack() : VEnemyAttack(false), nb_flies(0), timer(0)
{
}
FROGGITATTACK::~FroggitFlyAttack()
{
}

void FROGGITATTACK::regen_fly(unsigned index)
{
    flies_x[index] = (rand() % 80) * 8;
    flies_y[index] = 68 * 8;
    flies_dx[index] = rand() % 8;
    flies_dy[index] = -4;
    // if(flies_x[index] > 40)
    // 	flies_dx[index] = -flies_dx[index];
    flies_stop_timer[index] = 60;
    flies_stopped[index] = false;
}

void FROGGITATTACK::update(unsigned dt)
{
    timer += dt;
    // pop a new fly
    if (timer < (is_combined_attack ? nb_max_flies / 2 : nb_max_flies) * 30 &&
        (timer % 30) == 0)
    {
        regen_fly(nb_flies);
        ++nb_flies;
    }
    for (unsigned i = 0; i < nb_flies; ++i)
    {
        signed &fly_x = flies_x[i], &fly_y = flies_y[i];
        signed &fly_dx = flies_dx[i], &fly_dy = flies_dy[i];
        bool &fly_stopped = flies_stopped[i];
        if (!fly_stopped)
        {
            fly_x += fly_dx;
            fly_y += fly_dy;
            if (fly_y / 8 == 40)
            {
                fly_stopped = true;
            }
            if ((fly_y / 8) < -3 || (fly_x / 8) < -3 || (fly_x / 8) > 80)
            {
                regen_fly(i);
            }
        }
        else
        {
            signed &fly_stop_timer = flies_stop_timer[i];
            fly_stop_timer -= dt;
            if (fly_stop_timer <= 0)
            {
                fly_stopped = false;
                fly_dx = rand() % 8;
                if (fly_x > 0)
                    fly_dx = -fly_dx;
            }
        }
    }
}

void FROGGITATTACK::render(unsigned dt)
{
    for (int i = 0; i < nb_flies; ++i)
    {
        signed fx = (flies_x[i]) / 8;
        signed fy = (flies_y[i]) / 8;
        if (fy > 60)
            continue;
        Graphics::put_sprite_clipping(*WalrusRPG::Undertale::tex_froggit_bullet, fx + 121,
                                      fy + 128, {0, 0, 3, 3}, {121, 128, 80, 60});
    }
}

uint16_t FROGGITATTACK::calculate_damage(uint16_t chara_hp)
{
    if (chara_hp > 10)
        return 5;
    else
        return 3;
}

bool FROGGITATTACK::check_collision(WalrusRPG::Utils::Rect &player_hitbox)
{
    for (int i = 0; i < nb_flies; ++i)
    {
        // 1px sized hit-box because screw the rules!
        signed fx = (flies_x[i]) / 8 + 1;
        signed fy = (flies_y[i]) / 8 + 1;
        if (fx >= player_hitbox.x && fx <= player_hitbox.x + player_hitbox.width &&
            fy >= player_hitbox.y && fy <= player_hitbox.y + player_hitbox.height)
            return true;
    }
    return false;
}
