#include <cstdlib>
#include "UndertaleBattleStateMachine.h"
#include "BattleMainMenu.h"
#include "BattleEnemyAttack.h"
#include "BattleTextTransition.h"
#include "Froggit.h"
#include "utility/Rect.h"
#include "utility/misc.h"
#include "Item.h"
#include "attacks/FroggitFlyAttack.h"
#include <cstdio>
#include <cmath>

#define FROGGIT WalrusRPG::Undertale::Enemies::Froggit
using WalrusRPG::Undertale::Act;
using WalrusRPG::Undertale::doable_acts_e;
using WalrusRPG::Undertale::doable_acts;
using WalrusRPG::Undertale::IdleText;
using WalrusRPG::Undertale::Item;
using WalrusRPG::Undertale::BattleStates::BattleMainMenu;
using WalrusRPG::Undertale::BattleStates::BattleEnemyAttack;
using WalrusRPG::Undertale::BattleStates::BattleTextTransition;
using WalrusRPG::Undertale::EnemyAttacks::FroggitFlyAttack;
using WalrusRPG::Graphics::Texture;
using WalrusRPG::Graphics::Pixel;
using WalrusRPG::Utils::Rect;
using WalrusRPG::Animator;
using WalrusRPG::Animation;


static IdleText text[] = {
    // Encounter
    {"* Froggit attacks you!", "", ""},
    {"* Froggit hopped close!", "", ""},
    // Neutral
    {"* Froggit hops to and fro.", "", ""},
    {"* You are intimidated by", "Froggit's raw strength.", ""},
    {"* You are intimidated by", "Froggit's raw strength.", "Only kidding."},
    {"* Froggit doesn't seem to", "know why it's here. ", ""},
    {"* The battlefield is filled", "with the smell of mustard seed.", ""},
    // Spare
    {"* Froggit seems reluctant to fight", "you.", ""},
    // Compliment
    {"* Froggit didn't understand what", "you said, but was flattered anyway.", ""},
    // Threaten
    {"* Froggit didn't understand what", "you said, but was scared anyway.", ""},
    // Check
    {"Att: 4, Def: 5", "A frog.", "Or is it?"},
    // Low HP
    {"* Froggit is trying to run away.", "", ""}

};

static const char *thoughts[] = {
    "Ribbit, ribbit.",
    "Croak, croak.",
    "Hop, hop.",
    "Meow.",
    "(Blushes deeply.) Ribbit...",
    "Shiver, shiver."
};

static doable_acts_e froggit_acts[] = {doable_acts_e::CHECK, doable_acts_e::THREATEN,
                                       doable_acts_e::COMPLIMENT};

FROGGIT::Froggit(UndertaleBattleStateMachine &machine, unsigned cell_x, unsigned cell_y)
    : VEnemy(*WalrusRPG::Undertale::tex_froggit, "Froggit", true, machine, 30, 4, 4,
             cell_x, cell_y),
      last_player_action(0), time_spent(0), animator(),
      renderer(*WalrusRPG::Undertale::tex_froggit), head_offset(), total_time(0),
      current_time(0), current_frame(0), current_thought(0), shaking(false), shaking_timer(0)
{
    current_line = rand() % 2;
    current_thought = rand()%4;

    tinystl::vector<Frame> head_frames;
    head_frames.push_back({0, 72});
    head_frames.push_back({1, 72});
    animator.add_animation(0, {head_frames, true, 0});
    renderer.add_sprite(0, {0, 0, 51, 30});
    renderer.add_sprite(1, {0, 30, 51, 30});

    tinystl::vector<Frame> body_frames;
    body_frames.push_back({2, 96});
    body_frames.push_back({3, 96});
    animator.add_animation(1, {body_frames, true, 0});
    renderer.add_sprite(2, {51, 0, 46, 24});
    renderer.add_sprite(3, {51, 24, 46, 24});

    head_offset.push_back({-22, -29, 18}); // 1
    head_offset.push_back({-23, -29, 21}); // 2
    head_offset.push_back({-23, -30, 3});  // 3
    head_offset.push_back({-24, -30, 9});  // 4
    head_offset.push_back({-24, -31, 12}); // 5
    head_offset.push_back({-25, -32, 24}); // 6
    head_offset.push_back({-25, -32, 18}); // 7
    head_offset.push_back({-26, -31, 15}); // 8
    head_offset.push_back({-27, -31, 6});  // 9
    head_offset.push_back({-26, -31, 15}); // 10
    head_offset.push_back({-25, -31, 15}); // 11
    head_offset.push_back({-24, -31, 12}); // 12
    head_offset.push_back({-23, -31, 12}); // 13
    head_offset.push_back({-22, -31, 6});  // 14
    head_offset.push_back({-22, -30, 12}); // 15
    head_offset.push_back({-21, -30, 18}); // 16


    for (auto it = head_offset.begin(); it != head_offset.end(); it++)
        total_time += it->duration;
}

FROGGIT::~Froggit()
{
}

IdleText &FROGGIT::get_lines()
{
    if (hp <= 5)
        return text[11];
    else
     return text[current_line];
}

const char *FROGGIT::get_bubble()
{
    return thoughts[current_thought];
}


void FROGGIT::reset()
{
    VEnemy::reset();
}

void FROGGIT::on_act(Act &act)
{
    last_player_action = 2;
    current_thought = rand()%4;
    switch (act.get_id())
    {
        case CHECK:
            machine.set_state(new BattleTextTransition(machine, text[10],
                                                       new BattleEnemyAttack(machine)));
            current_line = rand() % 5 + 2;
            break;
        case THREATEN:
            machine.set_state(new BattleTextTransition(machine, text[9],
                                                       new BattleEnemyAttack(machine)));
            current_line = 9;
            spareable = true;
            current_thought = 4;
            break;
        case COMPLIMENT:
        default:
            machine.set_state(new BattleTextTransition(machine, text[8],
                                                       new BattleEnemyAttack(machine)));
            current_line = 8;
            spareable = true;
            current_thought = 4;
            break;
            // break;
    }
    if (spareable)
    {
        current_line = 7;
    }
}
void FROGGIT::on_attack(unsigned damage)
{
    UNUSED(damage);
    current_thought = rand()%4;
    last_player_action = 1;
    current_line = rand() % 5 + 2;
    if (hp >= damage)
        hp -= damage;
    else {
        hp = 0;
        Rect center({static_cast<int>(6 + 26 + 51 * cell_x), static_cast<int>(4 + 58 * cell_y + 29)});
        for(unsigned i = 0; i < 200; i++){
            WalrusRPG::Undertale::Particle p(center.x+rand()%33-16, center.y+rand()%33-16, rand()%3-1, -(rand()%8), 1, 40+rand()%20, Graphics::White);
            machine.parts.push_back(p);
        }
    }
    if (hp <= hp_max / 4)
    {
        spareable = true;
        current_line = 7;
    }
    shaking = true;
    shaking_timer = 30;
}

void FROGGIT::on_item(Item &item)
{
    UNUSED(item);
    current_thought = rand()%4;
    last_player_action = 3;
}


void FROGGIT::on_spare()
{
    if (spareable && !spared && !is_dead())
    {
        spared = true;
        Rect center({static_cast<int>(6 + 26 + 51 * cell_x), static_cast<int>(4 + 58 * cell_y + 29)});
        for(unsigned i = 0; i < 20; i++){
            WalrusRPG::Undertale::Particle p(center.x+0, center.y+0, rand()%15-7, rand()%15-3, 4, 40+rand()%20, Graphics::Gray);
            machine.parts.push_back(p);
        }
    }
}

void FROGGIT::update(unsigned dt)
{
    if (can_attack())
    {
        animator.update(dt);
        time_spent += dt;
        // offset
        current_time += dt;
        while (current_time > head_offset[current_frame].duration)
        {
            current_time -= head_offset[current_frame].duration;
            current_frame = (current_frame + 1) % head_offset.size();
        }
    }
    if (shaking_timer < dt)
        shaking = false;
    else
        shaking_timer -= dt;
}

void FROGGIT::render(Rect &center)
{
    if (is_dead() && !shaking)
        return;
    Offset &head_pos = head_offset[current_frame];
    Rect head_position{static_cast<int>(center.x + head_pos.x), static_cast<int>(center.y + head_pos.y),
                       0, 0};
    Rect body_position{static_cast<int>(center.x - 46 / 2), center.y, 0, 0};

    if (shaking)
    {
        signed offset = sin((shaking_timer / 30.) * M_PI * 2) * 4.;
        head_position.x += offset;
        body_position.x += offset;
    }

    if (spared)
    {
        renderer.render(animator.get_animation_frame(1), body_position, Graphics::Gray);
        renderer.render(animator.get_animation_frame(0), head_position, Graphics::Gray);
    }
    else
    {
        renderer.render(animator.get_animation_frame(1), body_position);
        renderer.render(animator.get_animation_frame(0), head_position);
    }
}
void FROGGIT::render(Rect &center, const WalrusRPG::Graphics::Pixel &tint)
{
    if (is_dead() && !shaking)
        return;
    Offset &head_pos = head_offset[current_frame];
    Rect head_position{static_cast<int>(center.x + head_pos.x), static_cast<int>(center.y + head_pos.y),
                       0, 0};
    Rect body_position{static_cast<int>(center.x - 46 / 2), static_cast<int>(center.y), 0, 0};
    if (spared)
    {
        Pixel tint2(tint.r << 2, tint.g << 1, tint.b << 2);
        renderer.render(animator.get_animation_frame(1), body_position, tint2);
        renderer.render(animator.get_animation_frame(0), head_position, tint2);
    }
    else
    {
        renderer.render(animator.get_animation_frame(1), body_position, tint);
        renderer.render(animator.get_animation_frame(0), head_position, tint);
    }
}

void FROGGIT::select_attack()
{
    current_line = rand() % 5 + 2;
    if (selected_attack != nullptr)
        delete selected_attack;
    selected_attack = new FroggitFlyAttack();
}

doable_acts_e *FROGGIT::get_doable_acts()
{
    return froggit_acts;
}
